import { ABC } from "../deps.ts";
import { extractPackagePath, getFile } from "../utils.ts";

const MAX_FIXED_CACHE_AGE = 60 * 60 * 24 * 365;
const MAX_FLOATING_CACHE_AGE = 60 * 30;

export default function raw(g: ABC.Group) {
   g.get("/:package/*path", async (ctx) => {
      console.log(ctx.params, ctx.path);
      let [packageName, packageVersion] = extractPackagePath(
         ctx.params.package
      );

      const E404 = () => {
         ctx.response.status = 404;
         ctx.response.body = "// Not found!";
      };

      const result = await getFile(
         packageName,
         packageVersion,
         ctx.params.path
      );

      if (packageVersion && result) {
         ctx.response.headers.set(
            "cache-control",
            "public, max-age=" + MAX_FIXED_CACHE_AGE
         );
      } else {
         ctx.response.headers.set(
            "cache-control",
            "no-cache, max-age=" + MAX_FLOATING_CACHE_AGE
         );
      }

      if (!result) return E404();

      ctx.response.headers.set("e-tag", result.etag);
      return result.data;
   });
}
